// mot_changer.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "mot_changer.h"

HANDLE         hComm = NULL;        // global com port handle
FILE           *logFile = NULL;     // global handle on log file
unsigned long  sleep = 200;         // default amount of time to sleep between data sends
TCHAR          port[10];            // name of the COM port our tuner is on.

/**
 * Entry point for this program.
 * It takes a channel number as input and changes the channel.
 * @param argc - the number of arguments specified
 * @param argv - the actual arguments
 * @return - always returns zero
 */
int _tmain(int argc, _TCHAR* argv[])
{
   size_t chanLen = _tcslen(argv[1]);

   ///////////////////
   // Open log file
   logFile = fopen( ".\\mot_changer.out", "a+" );
   printHeader( argc, argv );

   //////////////////////////////
   // Get settings
   TCHAR* tmp = getSetting( _T("port") );
   memset( port, '\0', sizeof(port) );
   _tcsncpy( port, tmp, sizeof(tmp) );
   tmp = getSetting( _T("sleepTime") );
   sleep = _ttol( tmp );

   ////////////////////////////////////
   // Initialize the tuner
   InitializeTuner();

   ////////////////////////////////////
   // Loop through each digit and send
   for( int i = 0; i < (int)chanLen; i++ )
   {
      int digit = argv[1][i] - 48;
      _ftprintf( logFile, _T("digit %d of channel: %d\n"), i+1, digit );

      // Send the command
		SendCommand( chanlookup[digit] );
		Sleep( sleep );
	}

   ////////////////////////////////////
   // Send the enter command
	//SendCommand( enter );

   ////////////////////////////////////
   // Shut down the tuner
   ShutdownTuner();

   // Close the log file
   fclose( logFile );

	return 0;
}

// build and return the data string based on the command passed
byte* BuildData( byte* cmd )
{
	for( int i = 0; i < 3; i++ )
		theData[28+i] = cmd[i];

	return theData;
}

/**
 * SendData - sends the specified data over the comm port
 * @param data - The data to send
 * @param size - The size of the data being sent
 * @return a boolean indicating if the data was sent or not
 */
bool SendData( byte *data, DWORD size )
{
	OVERLAPPED osWrite = {0};
	DWORD dwWritten;
	bool fRes;

	// Create this writes OVERLAPPED structure hEvent.
	osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (osWrite.hEvent == NULL)
		// Error creating overlapped event handle.
		return false;

	// Issue write.
	if (!WriteFile(hComm, data, size, &dwWritten, &osWrite))
	{
		if (GetLastError() != ERROR_IO_PENDING)
		{ 
			// WriteFile failed, but it isn't delayed. Report error and abort.
			fRes = false;
		}
		else {
			// Write is pending.
			if (!GetOverlappedResult(hComm, &osWrite, &dwWritten, TRUE))
				fRes = false;
			else
				// Write operation completed successfully.
				fRes = true;
		}
	}
	else
		// WriteFile completed immediately.
		fRes = true;

	CloseHandle(osWrite.hEvent);
	return fRes;
}

/**
 * SendCommand - builds a command then sends it over the comm port
 * @param command - The command to send
 * @return a boolean indicating if the data was sent or not
 */
bool SendCommand( byte* command )
{
	BuildData( command );
	return SendData( theData, theDataSize );
}

/**
 * Initializes the tuner
 * call before sending any command
 */
bool InitializeTuner()
{
	// try getting com port from ini file or default to COM1
	//TCHAR port[10];
   //memset(port, '\0', sizeof(port) );
   //_tcsncpy( port, _T("COM1"), _tcslen(_T("COM1")) );
   _ftprintf( logFile, _T("Port = %s\n"), port );

	hComm = CreateFile(
					port,  
					GENERIC_READ | GENERIC_WRITE, 
					0, 
					0, 
					OPEN_EXISTING,
					0,
					0 );

	if( hComm == INVALID_HANDLE_VALUE )
	{
      _ftprintf( logFile, _T("DCT2xxx Serial Tuner cannot open port: %s\n"), port );
		return false;
	}

	// setup the serial port
	DCB dcb;
	FillMemory(&dcb, sizeof(dcb), 0);
	dcb.DCBlength = sizeof(dcb);
	if( !BuildCommDCB(_T("9600,N,8,1"), &dcb) ) //baud=9600 parity=N data=8 stop=1
	{
      _ftprintf( logFile, _T("Couldn't build the DCB. Usually a problem a problem with the communications specification string.\n") );
		CloseHandle( hComm );
		hComm = NULL;
		return false;
	}
	dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
	SetCommState( hComm, &dcb );

   _ftprintf( logFile, _T("DCT2xxx Serial Tuner opened a port to: %s\n"), port );

	return true;
}

/**
 * Shuts down the tuner
 */
void ShutdownTuner()
{
	// close comm port
	if( hComm != NULL )
	{
      _ftprintf( logFile, _T("Closing port %s\n\n\n"), port );
		CloseHandle( hComm );
		hComm = NULL;
	}
}

/**
 * Prints a header to our log file before perfoming any operations
 * @param argc - number of parameters in argv
 * @param argv - Parameters supplied to the running program
 */
void printHeader( int argc, _TCHAR* argv[] )
{
   TCHAR buff[255];
   time_t currTime;
   char* pszCurrTime;
   size_t chanLen = _tcslen(argv[1]);

   memset( buff, '\0', sizeof(buff) );
   _stprintf( buff, _T("| Number of digits to send to DCT2xxx: %d\n"), chanLen );

   // Print a nice row of '=' chars based on length of longest string in header
   for( int i = 0; i < (int)_tcslen(buff); i++ )
   {
      _ftprintf( logFile, _T("=") );
   }
   _ftprintf( logFile, _T("====\n") );

   // Print Header contents
   currTime = time( NULL );
   pszCurrTime = ctime(&currTime);
   pszCurrTime[strlen(pszCurrTime)-1] = '\0';
   fprintf( logFile, "| %s\n", pszCurrTime );
   _ftprintf( logFile, _T("| Changing to channel %s\n"), argv[1]);
   _ftprintf( logFile, buff );

   // Print a nice row of '=' chars based on length of longest string in header
   for( int i = 0; i < (int)_tcslen(buff); i++ )
   {
      _ftprintf( logFile, _T("=") );
   }
   _ftprintf( logFile, _T("====\n") );
}

/**
 * Gets a setting from the ini file
 * @param name - The name of the parameter to retrieve
 * @return The value of the parameter requested
 */
TCHAR* getSetting( TCHAR* name )
{
   static TCHAR retval[MAX_PATH];

   if( GetPrivateProfileString(  _T("Settings"),
                                 name,
                                 _T(""),
                                 retval,
                                 sizeof(retval),
                                 _T(".\\mot_changer.ini") ) > 0 )
      return retval;

   return NULL;

}
